import unittest
from infi.traceback import pretty_traceback_and_exit_decorator
from infi.traceback import set_truncation_limit

@pretty_traceback_and_exit_decorator
def main():
    local_variable = None
    raise NotImplementedError()

@pretty_traceback_and_exit_decorator
def system_exit():
    raise SystemExit(3)

class RaisingRepr(object):
    def __repr__(self):
        raise Exception()

class TestCase(unittest.TestCase):
    def test_safe_repr(self):
        from infi.traceback import safe_repr
        obj = RaisingRepr()
        string = safe_repr(obj)

    def test_main(self):
        with self.assertRaises(SystemExit):
            main()

    def test_system_exit(self):
        try:
            system_exit()
        except SystemExit, error:
           self.assertEquals(error.args, (3, ))
           
    def test_truncated_repr(self):
        from StringIO import StringIO
        import sys
        @pretty_traceback_and_exit_decorator
        def func_with_variable():
            long_variable = "a very long string that we will truncate to 10 chars"
            raise Excetion()
        old_stderr = sys.stderr
        sys.stderr = StringIO()
        set_truncation_limit(10)
        try:
            with self.assertRaises(SystemExit):
                func_with_variable()
            sys.stderr.seek(0)
            output = sys.stderr.read()
        finally:
            sys.stderr = old_stderr
        self.assertIn("long_variable", output)
        self.assertIn("func_with_variable", output)
        self.assertIn("repr truncated", output)
        self.assertNotIn("10 chars", output)


if __name__ == "__main__":
    main()
    



__import__("pkg_resources").declare_namespace(__name__)

import sys
import nose
import mock
import linecache
import types
import traceback
import mock
import infi.pyutils.contexts
import infi.pyutils.patch
import infi.exceptools

truncate_repr = None

class NosePlugin(nose.plugins.Plugin):
    """better tracebacks"""
    name = 'infi-traceback'

    def __init__(self):
        super(NosePlugin, self).__init__()
        self.active_context = None

    def help(self):
        return "Print better tracebacks"

    def startContext(self, context):
        if self.active_context is not None:
            return
        self.active_context = context
        self.traceback_context = traceback_context()
        self.traceback_context.__enter__()

    def stopContext(self, context):
        if not self.active_context is context:
            return
        self.active_context = None
        self.traceback_context.__exit__(None, None, None)

@infi.pyutils.contexts.contextmanager
def traceback_context():
    with infi.pyutils.patch.patch(traceback, "format_tb", format_tb), \
         infi.pyutils.patch.patch(traceback, "print_tb", print_tb), \
         infi.pyutils.patch.patch(traceback, "format_exception", format_exception), \
         infi.pyutils.patch.patch(traceback, "print_exception", print_exception):
        yield

def traceback_decorator(func):
    @infi.pyutils.contexts.wraps(func)
    def callee(*args, **kwargs):
        with traceback_context():
            return func(*args, **kwargs)
    return callee

def pretty_traceback_and_exit_decorator(func):
    @infi.pyutils.contexts.wraps(func)
    def callee(*args, **kwargs):
        with traceback_context():
            import traceback
            try:
                return func(*args, **kwargs)
            except SystemExit:
                traceback.print_exc()
                raise
            except:
                traceback.print_exc()
                raise infi.exceptools.chain(SystemExit(1))
    return callee

# Taken from Python 2.7.2 traceback module

def format_tb(tb, limit=None):
    """A shorthand for 'format_list(extract_stack(f, limit))."""
    return format_list(extract_tb(tb, limit))

def print_tb(tb, limit=None, file=None):
    """Print up to 'limit' stack trace entries from the traceback 'tb'.

    If 'limit' is omitted or None, all entries are printed.  If 'file'
    is omitted or None, the output goes to sys.stderr; otherwise
    'file' should be an open file or file-like object with a write()
    method.
    """
    if file is None:
        file = sys.stderr
    if limit is None:
        if hasattr(sys, 'tracebacklimit'):
            limit = sys.tracebacklimit
    file.write('\n'.join(format_tb(tb, limit)) + '\n')

def extract_tb(tb, limit=None):
    if limit is None:
        if hasattr(sys, 'tracebacklimit'):
            limit = sys.tracebacklimit
    list = []
    n = 0
    while tb is not None and (limit is None or n < limit):
        f = tb.tb_frame
        lineno = tb.tb_lineno
        co = f.f_code
        filename = co.co_filename
        name = co.co_name
        _locals = tb. tb_frame.f_locals
        linecache.checkcache(filename)
        line = linecache.getline(filename, lineno, f.f_globals)
        if line: line = line.strip()
        else: line = None
        list.append((filename, lineno, name, line, _locals))
        tb = tb.tb_next
        n = n + 1
    return list

def set_truncation_limit(limit):
    global truncate_repr
    assert limit is None or (isinstance(limit, int) and limit > 0)
    truncate_repr = limit

def safe_repr(obj):
    try:
        res = repr(obj)
        if truncate_repr is not None and len(res) > truncate_repr:
            res = "<repr truncated: %s>" % (object.__repr__(obj), )
    except:
        res = "<repr fallback: %s>" % (object.__repr__(obj), )
    return res

def format_list(extracted_list):
    list = []
    for filename, lineno, name, line, _locals in extracted_list:
        item = '  File "%s", line %d, in %s\n' % (filename, lineno, name)
        if line:
            item = item + '    %s\n' % line.strip()
        if _locals:
            item = item + '  Local variables:\n'
            for key, value in _locals.items():
                item = item + '    %r: %s\n' % (key, safe_repr(value))
        list.append(item)
    return list

def format_exception(etype, value, tb, limit = None):
    """Format a stack trace and the exception information.

    The arguments have the same meaning as the corresponding arguments
    to print_exception().  The return value is a list of strings, each
    ending in a newline and some containing internal newlines.  When
    these lines are concatenated and printed, exactly the same text is
    printed as does print_exception().
    """
    import traceback
    if tb:
        list = ['Traceback (most recent call last):\n']
        list = list + format_tb(tb, limit)
    else:
        list = []
    list = list + traceback.format_exception_only(etype, value)
    return list

def print_exception(etype, value, tb, limit=None, file=None):
    """Print exception up to 'limit' stack trace entries from 'tb' to 'file'.

    This differs from print_tb() in the following ways: (1) if
    traceback is not None, it prints a header "Traceback (most recent
    call last):"; (2) it prints the exception type and value after the
    stack trace; (3) if type is SyntaxError and value has the
    appropriate format, it prints the line where the syntax error
    occurred with a caret on the next line indicating the approximate
    position of the error.
    """
    import traceback
    if file is None:
        file = sys.stderr
    if tb:
        traceback._print(file, 'Traceback (most recent call last):')
        print_tb(tb, limit, file)
    lines = traceback.format_exception_only(etype, value)
    for line in lines:
        traceback._print(file, line, '')



__version__ = "0.3.12"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = 'develop'
__git_remote_tracking_branch__ = 'origin/develop'
__git_remote_url__ = 'git@git.infinidat.com:host-opensource/infi.traceback.git'
__git_head_hash__ = 'd1857867602a5c28663e810d1df2c503148deef4'
__git_head_subject__ = 'HOSTDEV-2201 not using mock anymore'
__git_head_message__ = ''
__git_dirty_diff__ = ''



SETUP_INFO = dict(
    name = 'infi.traceback',
    version = '0.3.12',
    author = 'Guy Rozendorn',
    author_email = 'guy@rzn.co.il',

    url = 'https://github.com/Infinidat/infi.traceback',
    license = 'PSF',
    description = """better tracebacks""",
    long_description = """Tracebacks in Python are missing some useful debugging information, such is locals() in each stack frame
This module provides several mechanisms for better tracebacks:
* traceback_context. A context that in patches standard's library traceback module to print better tracebacks
* traceback_decorator. A decorator that calls the decorated method inside the traceback_context
* Nose Plugin. The plugin, enabled with '--with-infi-traceback', prints a better traceback for errors and failures
* pretty_traceback_and_exit_decorator. A decorator for console script entry points that prints exceptions and raises SystemExit""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Python Software Foundation License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = [
'infi.exceptools',
'infi.pyutils',
'nose',
'setuptools'
],
    namespace_packages = ['infi'],

    package_dir = {'': 'src'},
    package_data = {'': []},
    include_package_data = True,
    zip_safe = False,

    entry_points = dict(
        console_scripts = [],
        gui_scripts = [],
        ),
)

if SETUP_INFO['url'] is None:
    _ = SETUP_INFO.pop('url')

SETUP_INFO['entry_points']['nose.plugins'] = ['infi.traceback = infi.traceback:NosePlugin']

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)

if __name__ == '__main__':
    setup()

